import unittest

from lxml import etree as et

from eml.resources import EMLLicense
from test_xml.test_xml import TestXML


class TestLicence(TestXML):
    def test_parse_simple(self):
        text_xml = """
<licensed>
    <licenseName>Apache License 2.0</licenseName>
</licensed>
        """
        licence = EMLLicense.from_string(text_xml)
        self.assertEqual("Apache License 2.0", licence.name, "Error on parsing name")
        self.assertIsNone(licence.url, "Error on parsing url")
        self.assertIsNone(licence.id, "Error on parsing identifier")
        self.assertEqualTree(et.fromstring(text_xml), licence.to_element(), "Error on to element")

    def test_parse_empty_name(self):
        text_xml = """
<licensed>
    <licenseName/>
</licensed>
        """
        licence = EMLLicense.from_string(text_xml)
        self.assertEqual("", licence.name, "Error on parsing name")
        self.assertIsNone(licence.url, "Error on parsing url")
        self.assertIsNone(licence.id, "Error on parsing identifier")
        self.assertEqualTree(et.fromstring(text_xml), licence.to_element(), "Error on to element")

    def test_parse_complete(self):
        text_xml = """
<licensed>
    <licenseName>Apache License 2.0</licenseName>
    <url>https://spdx.org/licenses/Apache-2.0.html</url>
    <identifier>Apache-2.0</identifier>
</licensed>
        """
        licence = EMLLicense.from_string(text_xml)
        self.assertEqual("Apache License 2.0", licence.name, "Error on parsing name")
        self.assertEqual("https://spdx.org/licenses/Apache-2.0.html", licence.url, "Error on parsing url")
        self.assertEqual("Apache-2.0", licence.id, "Error on parsing identifier")
        self.assertEqualTree(et.fromstring(text_xml), licence.to_element(), "Error on to element")

    def test_parse_none(self):
        self.assertIsNone(EMLLicense.parse(None, {}), "Licence parsed from nowhere")


if __name__ == '__main__':
    unittest.main()



import unittest
from typing import Callable

from lxml import etree as et

from dwca.terms import OutsideTerm
from test_xml.test_xml import TestXML


class TestOutsideTerm(TestXML):
    def test_simple_term(self):
        term_xml = """
<field index="0" term="http://example.org/terms/example"/>
        """
        term = OutsideTerm.from_string(term_xml)
        self.assertEqual("example", term.name, "Error parsing name.")
        self.assertEqual("VARCHAR", term.sql_type, "Wrong SQL type.")
        self.assertEqual(0, term.index, "Index wrongly parsed")
        self.assertIsNone(term.default, "Default value from nowhere")
        self.assertIsNone(term.vocabulary, "Vocabulary from nowhere")
        self.assertEqual("http://example.org/terms/example", term.uri, "URI wrongly parsed")
        self.assertEqualTree(et.fromstring(term_xml), term.to_element(), "Error on to element")

    def test_complete_term(self):
        term_xml = """
<field
    index="0"
    term="http://example.org/terms/example"
    default="Default value"
    vocabulary="http://example.org/vocabulary/example"
/>
        """
        term = OutsideTerm.from_string(term_xml)
        self.assertEqual("example", term.name, "Error parsing name.")
        self.assertEqual("VARCHAR", term.sql_type, "Wrong SQL type.")
        self.assertEqual(0, term.index, "Index wrongly parsed")
        self.assertEqual("Default value", term.default, "Default wrongly parsed")
        self.assertEqual("http://example.org/vocabulary/example", term.vocabulary, "Vocabulary wrongly parsed")
        self.assertEqual("http://example.org/terms/example", term.uri, "URI wrongly parsed")
        self.assertEqualTree(et.fromstring(term_xml), term.to_element(), "Error on to element")

    def test_parse_invalid(self):
        self.assertRaises(
            ValueError,
            OutsideTerm.from_string,
            '<field index="OK" term="http://rs.tdwg.org/dwc/terms/scientificNameID"/>'
        )
        self.assertRaises(
            AssertionError,
            OutsideTerm.from_string,
            '<archive index="0" term="Valid Term"/>'
        )
        self.assertRaises(
            TypeError,
            OutsideTerm.from_string,
            '<field index="2"/>'
        )

    def test_repr(self):
        term_xml = """
        <field index="0" term="http://example.org/terms/example"/>
                """
        term = OutsideTerm.from_string(term_xml)
        self.assertEqual(
            "<Field [term=http://example.org/terms/example]>",
            repr(term),
            "Term wrongly parsed"
        )

    def test_parse_none(self):
        self.assertIsNone(
            OutsideTerm.parse(None, {}),
            "Parse term from nowhere"
        )

    def test_no_index(self):
        self.assertRaises(
            TypeError,
            OutsideTerm.from_string,
            """
<field index="0"/>
            """
        )

    def test_invalid_format(self):
        term = OutsideTerm(3, "http://example.org/terms/example")

        class A:
            pass

        term.TYPE = A
        self.assertRaisesRegex(
            TypeError, "automatic conversion",
            term.format, "a"
        )
        term.TYPE = Callable
        self.assertRaisesRegex(
            TypeError, "automatic conversion",
            term.format, "a"
        )


if __name__ == '__main__':
    unittest.main()



import unittest
from lxml import etree as et
from eml.types import ExtensionString
from test_xml.test_xml import TestXML


class TestExtensionString(TestXML):
    def test_extension(self):
        variable = ExtensionString("My variable")
        self.assertEqual("My variable", variable, "Not equal to string")
        self.assertEqual("My variable", str(variable), "Not equal to string")
        self.assertEqual("<ExtensionString (My variable)>", repr(variable), "Not equal to string")
        self.assertIsNone(variable.system, "System set incorrectly")

    def test_extension_system(self):
        variable = ExtensionString("My variable", system="http://gbif.org")
        self.assertEqual("My variable", variable, "Not equal to string")
        self.assertEqual(
            "<ExtensionString system='http://gbif.org' (My variable)>",
            repr(variable),
            "Not equal to string"
        )
        self.assertIsNotNone(variable.system, "System set incorrectly")
        self.assertEqual("http://gbif.org", variable.system, "System set incorrectly")

    def test_parse(self):
        self.assertIsNone(ExtensionString.parse(None, {}), "Parsing something")
        example_xml = """
<alternateIdentifier
    system="http://gbif.org"
>My variable</alternateIdentifier>
        """
        variable = ExtensionString.from_string(example_xml)
        self.assertEqual("My variable", variable, "Not equal to string")
        self.assertIsNotNone(variable.system, "System set incorrectly")
        self.assertEqual("http://gbif.org", variable.system, "System set incorrectly")

    def test_parse_no_system(self):
        example_xml = """
<alternateIdentifier>My variable</alternateIdentifier>
        """
        variable = ExtensionString.from_string(example_xml)
        self.assertEqual("My variable", variable, "Not equal to string")
        self.assertIsNone(variable.system, "System set incorrectly")

    def test_to_element(self):
        variable = ExtensionString("My variable")
        self.assertRaises(RuntimeError, variable.to_element)
        variable.set_tag("alternateIdentifier")
        expected = et.Element("alternateIdentifier")
        expected.text = "My variable"
        self.assertEqualTree(expected, variable.to_element(), "Error on element")

    def test_to_element_system(self):
        variable = ExtensionString("My variable", system="A system")
        self.assertRaises(RuntimeError, variable.to_element)
        variable.set_tag("alternateIdentifier")
        expected = et.Element("alternateIdentifier")
        expected.text = "My variable"
        expected.set("system", "A system")
        self.assertEqualTree(expected, variable.to_element(), "Error on element")

    def test_equal(self):
        variable = ExtensionString("My variable")
        self.assertEqual("My variable", variable, "Error on equal implementation")
        self.assertEqual(ExtensionString("My variable"), variable, "Error on equal implementation")
        self.assertEqual(variable, ExtensionString(variable), "Error on equal with initialization parameter")

    def test_not_equal(self):
        variable = ExtensionString("My variable")
        self.assertNotEqual(ExtensionString("Another variable"), variable, "Error on equal implementation")
        self.assertNotEqual(30, variable, "Error on equal implementation")


if __name__ == '__main__':
    unittest.main()



import unittest

from lxml import etree as et

from eml.types import ExtensionURI
from test_xml.test_xml import TestXML


class TestExtensionURI(TestXML):
    def test_extension(self):
        variable = ExtensionURI("http://a_link.org", "A label")
        self.assertEqual("http://a_link.org", variable, "Not equal to string")
        self.assertEqual("http://a_link.org", str(variable), "Not equal to string")
        self.assertEqual(
            "<ExtensionURI (http://a_link.org [label=A label])>",
            repr(variable), "Not equal to string"
        )
        self.assertEqual("A label", variable.label, "Label set incorrectly")

    def test_valid_uri(self):
        self.assertRaises(
            ValueError, ExtensionURI, "An invalid URL", "A label"
        )

    def test_parse(self):
        self.assertIsNone(ExtensionURI.parse(None, {}), "Parsing something")
        example_xml = """
<propertyURI
    label="A label"
>http://a_link.org</propertyURI>
        """
        variable = ExtensionURI.from_string(example_xml)
        self.assertEqual("http://a_link.org", variable, "Not equal to string")
        self.assertIsNotNone(variable.label, "Label set incorrectly")
        self.assertEqual("A label", variable.label, "Label set incorrectly")

    def test_to_element(self):
        variable = ExtensionURI("http://a_link.org", "A label")
        self.assertRaises(RuntimeError, variable.to_element)
        variable.set_tag("propertyURI")
        expected = et.Element("propertyURI")
        expected.set("label", "A label")
        expected.text = "http://a_link.org"
        self.assertEqualTree(expected, variable.to_element(), "Error on element")

    def test_equal(self):
        variable = ExtensionURI("http://a_link.org", "A label")
        self.assertEqual("http://a_link.org", variable, "Error on equal implementation")
        self.assertEqual(ExtensionURI("http://a_link.org", "A label"), variable, "Error on equal implementation")

    def test_not_equal(self):
        variable = ExtensionURI("http://a_link.org", "A label")
        self.assertNotEqual(
            ExtensionURI("http://a_link.org", "Another label"),
            variable, "Error on equal implementation"
        )
        self.assertNotEqual(
            ExtensionURI("http://another_link.org", "A label"),
            variable, "Error on equal implementation"
        )
        self.assertNotEqual(
            ExtensionURI("http://another_link.org", "Another label"),
            variable, "Error on equal implementation"
        )
        self.assertNotEqual(30, variable, "Error on equal implementation")


if __name__ == '__main__':
    unittest.main()



import os.path
import sys
import tempfile
import unittest
from unittest.mock import patch

import pandas as pd
from lxml import etree as et

from dwca.classes import Taxon
from dwca.terms import DWCLanguage, TaxonID
from test_dwca_classes.test_taxon_common import TestTaxonCommon
from xml_common.utils import Language

PATH = os.path.abspath(os.path.dirname(__file__))

orig_import = __import__


def import_mock_fuzzy(name, globals=None, locals=None, fromlist=(), level=0):
    if name == 'rapidfuzz':
        raise ImportError(f"No module named '{name}'")
    return orig_import(name, globals, locals, fromlist, level)


class TestTaxon(TestTaxonCommon):
    def setUp(self) -> None:
        self.nmap, self.taxon_xml, self.text = self.read_xml(os.path.join(PATH, os.pardir, "example_data", "meta.xml"))
        self.taxon = None
        return

    def tearDown(self) -> None:
        if self.taxon is not None and self.taxon.is_lazy():
            self.taxon.close()

    def test_name(self):
        taxon = Taxon.from_string(self.text)
        self.assertEqual("Taxon", taxon.name, "Name parsed incorrectly.")

    def test_add_field(self):
        super().__test_add_field__()

    def test_add_field_incorrect_index(self):
        super().__test_add_field_incorrect_index__()

    def test_add_field_data(self):
        self.read_pandas()
        df = self.taxon.polars
        self.assertEqual(47, len(df.columns), "Wrong number of initial columns")
        self.assertRaisesRegex(
            AttributeError, "language",
            getattr,
            self.taxon.__entries__[0],
            "language"
        )
        self.taxon.add_field(DWCLanguage(index=47, default=Language.ENG))
        self.assertEqual(
            Language.ENG,
            self.taxon.__entries__[0].language,
            "Language not write on all entries"
        )
        df = self.taxon.polars
        self.assertEqual(48, len(df.columns), "Wrong number of columns after set")
        self.assertEqual(48, len(self.taxon.fields), "Fields not set")
        self.text = self.text.replace(
            """<field index="46" default="EXAMPLE" term="http://rs.tdwg.org/dwc/terms/institutionCode"/>""",
            """<field index="46" default="EXAMPLE" term="http://rs.tdwg.org/dwc/terms/institutionCode"/>
    <field index="47" default="eng" term="http://purl.org/dc/elements/1.1/language"/>""")
        self.assertEqual(Language.ENG, self.taxon.polars.item(7, "language"), "Wrong set default on polars.")
        self.assertEqual(Language.ENG, self.taxon.__entries__[7].language, "Wrong set default on entries.")
        self.assertEqualTree(et.fromstring(self.text), self.taxon.to_element(), "Error on element conversion with new field")

    def test_add_field_lazy(self):
        self.read_pandas(lazy=True)
        self.assertNotIn(
            DWCLanguage(0).uri,
            self.taxon.fields,
            "Language in fields"
        )
        self.assertIn(
            TaxonID(0).uri,
            self.taxon.fields,
            "TaxonID not in fields"
        )
        df = self.taxon.polars
        self.assertEqual(47, len(df.columns), "Wrong number of initial columns")
        self.read_pandas(lazy=True)
        self.taxon.add_field(DWCLanguage(index=47, default=Language.ENG))
        df = self.taxon.polars
        self.assertEqual(48, len(df.columns), "Wrong number of columns after set")
        self.assertEqual(48, len(self.taxon.fields), "Fields not set")
        self.text = self.text.replace(
            """<field index="46" default="EXAMPLE" term="http://rs.tdwg.org/dwc/terms/institutionCode"/>""",
            """<field index="46" default="EXAMPLE" term="http://rs.tdwg.org/dwc/terms/institutionCode"/>
    <field index="47" default="eng" term="http://purl.org/dc/elements/1.1/language"/>""")
        self.assertEqual(Language.ENG.value, self.taxon.polars.item(7, "language"), "Wrong set default on polars.")
        self.assertEqualTree(et.fromstring(self.text), self.taxon.to_element(),
                             "Error on element conversion with new field")


    def test_parse(self):
        super().__test_parse__()

    def test_filter_kingdom_exception(self):
        super().__test_filter_kingdom_exception__()

    def test_get_parents(self):
        super().__test_get_parents__()

    def test_get_incorrect_parent(self):
        self.__test_get_incorrect_parent__()

    def test_get_synonyms(self):
        self.__test_get_synonyms__()

    def test_get_synonyms_names(self):
        super().__test_get_synonyms_names__()

    def test_get_incorrect_synonyms(self):
        self.__test_get_incorrect_synonyms__()

    def test_filter_kingdom(self):
        self.read_pandas()
        df = self.taxon.pandas
        length_df = len(df)
        kingdom_summary = df.groupby("kingdom").size().to_dict()
        print(f"Kingdom Summary: {kingdom_summary}", file=sys.stderr)
        self.assertEqual(length_df, len(self.taxon), "Original length of dataframe.")
        self.taxon.filter_by_kingdom(kingdom_summary.keys())
        self.assertEqual(length_df, len(self.taxon), "Filter by all kingdoms.")
        first_kingdom = list(kingdom_summary.keys())[0]
        self.taxon.filter_by_kingdom([first_kingdom])
        self.assertEqual(kingdom_summary[first_kingdom], len(self.taxon), "Filter by first kingdom.")

    def test_filter_phylum(self):
        self.read_pandas()
        df = self.taxon.pandas
        length_df = len(df)
        phylum_summary = df.groupby(["kingdom", "phylum"]).size().to_dict()
        kingdom_list = list()
        phylum_list = list()
        for candid in phylum_summary.keys():
            if candid[1] != "":
                phylum_list.append(candid[1])
            else:
                kingdom_list.append(candid[0])
        print(f"Phylum Summary: {phylum_summary}", file=sys.stderr)
        self.assertEqual(length_df, len(self.taxon), "Original length of dataframe.")
        self.taxon.filter_by_phylum(phylum_list)
        self.assertEqual(length_df, len(self.taxon), "Filter by all phylum.")
        self.taxon.filter_by_phylum([phylum_list[0]])
        self.assertEqual(
            phylum_summary[(kingdom_list[0], "")] + phylum_summary[(kingdom_list[0], phylum_list[0])],
            len(self.taxon),
            f"Incorrect filter by {phylum_list[0]}."
        )

    def test_filter_phylum_no_threshold(self):
        self.read_pandas()
        df = self.taxon.pandas
        length_df = len(df)
        phylum_summary = df.groupby(["kingdom", "phylum"]).size().to_dict()
        kingdom_list = list()
        phylum_list = list()
        for candid in phylum_summary.keys():
            if candid[1] != "":
                phylum_list.append(candid[1])
            else:
                kingdom_list.append(candid[0])
        print(f"Phylum Summary: {phylum_summary}", file=sys.stderr)
        self.assertEqual(length_df, len(self.taxon), "Original length of dataframe.")
        # Modified phylum 0
        # Change 2 letter
        phylum_as_list = list(phylum_list[0])
        phylum_as_list[4] = "Y"
        phylum_as_list.pop(3)
        phylum_list[0] = ''.join(phylum_as_list)
        self.taxon.filter_by_phylum(phylum_list, fuzzy_threshold=100)
        self.assertGreater(length_df, len(self.taxon), "Filter by all phylum with no threshold.")

    def test_filter_phylum_threshold(self):
        self.read_pandas()
        df = self.taxon.pandas
        length_df = len(df)
        phylum_summary = df.groupby(["kingdom", "phylum"]).size().to_dict()
        kingdom_list = list()
        phylum_list = list()
        for candid in phylum_summary.keys():
            if candid[1] != "":
                phylum_list.append(candid[1])
            else:
                kingdom_list.append(candid[0])
        print(f"Phylum Summary: {phylum_summary}", file=sys.stderr)
        self.assertEqual(length_df, len(self.taxon), "Original length of dataframe.")
        # Modified phylum 0
        # Change 2 letter
        phylum_as_list = list(phylum_list[0])
        phylum_as_list[4] = "Y"
        phylum_as_list.pop(3)
        phylum_list[0] = ''.join(phylum_as_list)
        self.taxon.filter_by_phylum(phylum_list, fuzzy_threshold=50)
        self.assertEqual(length_df, len(self.taxon), "Filter by all phylum with enough threshold.")
        self.taxon.filter_by_phylum(phylum_list, fuzzy_threshold=90)
        self.assertGreater(length_df, len(self.taxon), "Filter by all phylum with not enough threshold.")

    def test_filter_class(self):
        self.read_pandas()
        df = self.taxon.pandas
        length_df = len(df)
        class_summary = df.groupby(["kingdom", "phylum", "class"]).size().to_dict()
        kingdom_list = list()
        phylum_list = list()
        class_list = list()
        for candid in class_summary.keys():
            if candid[2] != "":
                class_list.append(candid[2])
            else:
                if candid[1] != "":
                    phylum_list.append(candid[1])
                else:
                    kingdom_list.append(candid[0])
        print(f"Class Summary: {class_summary}", file=sys.stderr)
        self.assertEqual(length_df, len(self.taxon), "Original length of dataframe.")
        self.taxon.filter_by_class(class_list)
        self.assertEqual(length_df, len(self.taxon), "Filter by all classes.")
        self.taxon.filter_by_class([class_list[2]])
        self.assertEqual(
            class_summary[(kingdom_list[0], "", "")] +
            class_summary[(kingdom_list[0], phylum_list[0], "")] +
            class_summary[(kingdom_list[0], phylum_list[0], class_list[1])] +
            class_summary[(kingdom_list[0], phylum_list[0], class_list[2])],
            len(self.taxon),
            f"Incorrect filter by {class_list[2]}."
        )

    def test_filter_order(self):
        self.read_pandas()
        df = self.taxon.pandas
        length_df = len(df)
        orders = list(filter(lambda x: x != "", pd.unique(df["order"])))
        order_names = ", ".join([f'"{order}"' for order in orders])
        print(f"Orders found: {order_names}", file=sys.stderr)
        self.assertEqual(length_df, len(self.taxon), "Original length of dataframe.")
        self.taxon.filter_by_order(orders)
        self.assertEqual(length_df, len(self.taxon), "Filter by all orders.")
        accepted = df[df["scientificName"] == "Rbaliuycliu"]["acceptedNameUsage"].iloc[0]
        with_order = sum(df["order"].isin(["Rbaliuycliu", "Xlhofowltm", accepted]))
        taxa_id = df[df["scientificName"].isin(["Rbaliuycliu", "Xlhofowltm", accepted])]["taxonID"]
        parents = self.taxon.get_parents(taxa_id)
        parents.update(self.taxon.all_synonyms(parents))
        parents = sum(df["taxonID"].isin(parents))
        print("Filtering by orders: Rbaliuycliu, Xlhofowltm", file=sys.stderr)
        print(f"Accepted orders: Rbaliuycliu, {accepted}", file=sys.stderr)
        self.taxon.filter_by_order(["Rbaliuycliu", "Xlhofowltm"])
        print(f"Expected: with order {with_order} and parents {parents}", file=sys.stderr)
        self.assertEqual(with_order + parents, len(self.taxon), "Incorrect filter.")

    def test_filter_family(self):
        self.read_pandas()
        df = self.taxon.pandas
        length_df = len(df)
        families = list(filter(lambda x: x != "", pd.unique(df["family"])))
        family_names = ", ".join([f'"{family}"' for family in families])
        print(f"Families found: {family_names}", file=sys.stderr)
        self.assertEqual(length_df, len(self.taxon), "Original length of dataframe.")
        self.taxon.filter_by_family(families)
        self.assertEqual(length_df, len(self.taxon), "Filter by all families.")
        accepted = df[df["scientificName"] == "Dxaougdpy"]["acceptedNameUsage"].iloc[0]
        with_family = sum(df["family"].isin(["Dxaougdpy", "Kvtgqwfmy", accepted]))
        taxa_id = df[df["scientificName"].isin(["Dxaougdpy", "Kvtgqwfmy", accepted])]["taxonID"]
        parents = self.taxon.get_parents(taxa_id)
        parents.update(self.taxon.all_synonyms(parents))
        parents = sum(df["taxonID"].isin(parents))
        print("Filtering by family: Dxaougdpy, Kvtgqwfmy", file=sys.stderr)
        print(f"Accepted families: Kvtgqwfmy, {accepted}", file=sys.stderr)
        self.taxon.filter_by_family(["Dxaougdpy", "Kvtgqwfmy"])
        print(f"Expected: with family {with_family} and parents {parents}", file=sys.stderr)
        self.assertEqual(with_family + parents, len(self.taxon), "Incorrect filter.")

    def test_filter_genus(self):
        self.read_pandas()
        df = self.taxon.pandas
        length_df = len(df)
        genera = list(filter(lambda x: x != "", pd.unique(df["genus"])))
        genus_names = ", ".join([f'"{genus}"' for genus in genera])
        print(f"Genera found: {genus_names}", file=sys.stderr)
        self.assertEqual(length_df, len(self.taxon), "Original length of dataframe.")
        self.taxon.filter_by_genus(genera)
        self.assertEqual(length_df, len(self.taxon), "Filter by all genera.")
        accepted = df[df["scientificName"] == "Hbqrtfopjuh"]["acceptedNameUsage"].iloc[0]
        with_genus = sum(df["genus"].isin(["Hbqrtfopjuh", "Vbpjkmfqd", accepted]))
        taxa_id = df[df["scientificName"].isin(["Hbqrtfopjuh", "Vbpjkmfqd", accepted])]["taxonID"]
        parents = self.taxon.get_parents(taxa_id)
        parents.update(self.taxon.all_synonyms(parents))
        parents = sum(df["taxonID"].isin(parents))
        print("Filtering by genus: Hbqrtfopjuh, Vbpjkmfqd", file=sys.stderr)
        print(f"Accepted genera: Vbpjkmfqd, {accepted}", file=sys.stderr)
        self.taxon.filter_by_genus(["Hbqrtfopjuh", "Vbpjkmfqd"])
        print(f"Expected: with genus {with_genus} and parents {parents}", file=sys.stderr)
        self.assertEqual(with_genus + parents, len(self.taxon), "Incorrect filter.")

    def test_filter_species(self):
        self.read_pandas()
        df = self.taxon.pandas
        length_df = len(df)
        self.assertEqual(length_df, len(self.taxon), "Original length of dataframe.")
        species_synonym = "Hqmjhacvb (Azvoxtzhwueu) gupfjmnf"
        variety = "Rtfkiaicpdng obzninluz var. cebwyzcqoy Glhnskwn"
        cultivar = "Rtfkiaicpdng (Abifwvxqn) gurqtwpof f. prczacpvtdtu 'xzhgezqpaorp'"
        synonyms = set()
        species_row = df[df["scientificName"] == species_synonym].iloc[0]
        accepted = species_row["acceptedNameUsageID"]
        synonyms.add(accepted)
        synonyms.add(species_row["taxonID"])
        first_parents = self.taxon.get_parents([accepted])
        print(f"For {species_synonym} expected {len(first_parents)} parents, accepted and synonym", file=sys.stderr)
        variety_id = df[df["scientificName"] == variety]["taxonID"].iloc[0]
        second_parents = self.taxon.get_parents([variety_id])
        synonyms.add(variety_id)
        print(f"For {variety} expected {len(second_parents)} parents and self", file=sys.stderr)
        cultivar_id = df[df["scientificName"] == cultivar]["taxonID"].iloc[0]
        third_parents = self.taxon.get_parents([cultivar_id])
        synonyms.add(cultivar_id)
        print(f"For {cultivar} expected {len(third_parents)} parents", file=sys.stderr)
        synonyms.update(self.taxon.all_synonyms(
            list(synonyms.copy()) +
            list(first_parents) +
            list(second_parents) +
            list(third_parents)
        ))
        other_synonyms = synonyms.difference({species_synonym, variety, cultivar})
        other_synonyms = other_synonyms.difference(set(first_parents))
        other_synonyms = other_synonyms.difference(set(second_parents))
        other_synonyms = other_synonyms.difference(set(third_parents))
        print(f"Synonyms found: {len(other_synonyms)}", file=sys.stderr)
        print(f"Total {len(synonyms)}", file=sys.stderr)
        self.taxon.filter_by_species([species_synonym, variety, cultivar])
        self.assertEqual(
            len(synonyms),
            len(self.taxon),
            "Filter by all genera."
        )

    def test_filter_phylum_exception(self):
        super().__test_filter_phylum_exception__()

    def test_none(self):
        super().__test_none__()

    def test_merge(self):
        super().__test_merge__()

    def test_missing_field(self):
        self.assertRaises(
            AssertionError,
            Taxon.from_string,
            """
<core>
    <files>
      <location>missing.txt</location>
    </files>
    <id index="0"/>
    <field index="0" term=""/>
    <field index="1" term=""/>
    <field index="3" term=""/>
</core>
            """
        )

    def test_field_twice(self):
        self.assertRaises(
            AssertionError,
            Taxon.from_string,
            """
<core>
    <files>
      <location>missing.txt</location>
    </files>
    <id index="0"/>
    <field index="0" term=""/>
    <field index="1" term=""/>
    <field index="3" term=""/>
    <field index="2" term=""/>
    <field index="3" term=""/>
    <field index="4" term=""/>
</core>
            """
        )

    def test_parse_invalid(self):
        self.assertRaises(
            AssertionError,
            Taxon.from_string,
            '<extension />'
        )
        self.assertRaises(
            KeyError,
            Taxon.from_string,
            """
<core-id>
    <field index="0" term=""/>
</core-id>
            """
        )

    def test_set_pandas(self):
        self.read_pandas()
        df = self.taxon.pandas
        self.assertEqual(163460, len(self.taxon), "Data incorrectly read.")
        self.assertEqual(163460, len(df), "DataFrame incorrectly read.")
        df = df[0:100]
        self.taxon.pandas = df
        self.assertEqual(100, len(df), "DataFrame incorrectly set.")
        self.assertEqual(100, len(self.taxon), "Data incorrectly set.")

    def test_write_file(self):
        self.read_pandas()
        plain_text = self.taxon.write_file()
        self.maxDiff = None
        with open(os.path.join(PATH, os.pardir, "example_data", "taxon.txt"), "r", encoding="utf-8") as file:
            for expected, actual in zip(file.read().split("\n"), plain_text.split("\n")):
                self.assertEqual(
                    expected,
                    actual,
                    "File written incorrectly."
                )

    def test_read_lazy(self):
        self.read_pandas(lazy=True)
        self.assertEqual(
            163460, len(self.taxon), "Data incorrectly read."
        )

    def test_close(self):
        with tempfile.NamedTemporaryFile(delete=True) as file:
            temp_folder = os.path.dirname(file.name)
            temp_files = os.listdir(temp_folder)
            temp_files.remove(os.path.basename(file.name))
        self.read_pandas(lazy=True)
        self.assertTrue(self.taxon.is_lazy(), "`lazy` did not set.")
        self.taxon.close()
        self.assertCountEqual(temp_files, os.listdir(temp_folder), "File was not deleted.")

    def test_set_core_field(self):
        self.__test_set_core_field__()

    def test_set_primary_key(self):
        self.__test_set_primary_key__()

    def test_sql_table(self):
        self.__test_sql_table__()

    @patch('builtins.__import__', side_effect=import_mock_fuzzy)
    def test_fuzzy_exception(self, mock_import):
        self.__test_fuzzy_exception__()


if __name__ == '__main__':
    unittest.main()



from __future__ import annotations

from enum import Enum


ABBREVIATIONS = {
    "es": "esp",
    "esp": "esp",
    "spanish": "esp",
    "en": "eng",
    "eng": "eng",
    "english": "eng",
}


class Language(Enum):
    """
    The language of the resource.

    Attributes
    ----------
    ESP : str
        Spanish language
    ENG : str
        English language
    """
    ESP = "Spanish"
    ENG = "English"

    @staticmethod
    def get_language(abbreviation: str) -> Language:
        for lang in Language:
            if lang.name.lower() == ABBREVIATIONS[abbreviation.lower()]:
                return lang
        raise NotImplementedError(f"{abbreviation} language not implemented yet")



from __future__ import annotations

from typing import Dict

from lxml import etree as et

from eml.resources import Resource


# TODO: Implement Software Resource (https://eml.ecoinformatics.org/schema/eml_xsd#eml_software)
class EMLSoftware(Resource):
    """
    EML Software Resource.

    Other Parameters
    ----------------
    **kwargs : :class:`eml.resources.resource.Resource` parameters.
        The parameters of every type of Resource.
    """
    PRINCIPAL_TAG = "software"
    def __init__(self, **kwargs) -> None:
        super().__init__(**kwargs)
        return

    @classmethod
    def get_referrer(cls, element: et.Element, nmap: Dict) -> EMLSoftware:
        """
        Generate an EML Software referencing another EML Software.

        Parameters
        ----------
        element : lxml.etree.Element
            XML element to parse with references another software.
        nmap : Dict
            Namespace.

        Returns
        -------
        EMLSoftware
            Object parsed that reference another software.
        """
        references = element.find("references", nmap)
        return EMLSoftware(
            _id=references.text,
            scope=cls.get_scope(element),
            system=element.get("system", None),
            referencing=True,
            references_system=references.get("system", None)
        )

    @classmethod
    def get_no_referrer(cls, element: et.Element, nmap: Dict) -> EMLSoftware:
        """
        Generate an EML software that do not reference another software.

        Parameters
        ----------
        element : lxml.etree.Element
            XML element to parse.
        nmap : Dict
            Namespace.

        Returns
        -------
        EMLSoftware
            Object parsed.
        """
        pass

    def to_element(self) -> et.Element:
        """
        Generate an XML `Element` from this object

        Returns
        -------
        lxml.etree.Element
            XML `Element` from this object
        """
        software_elem = super().to_element()
        software_elem = self._to_element_(software_elem)
        return software_elem



from __future__ import annotations

from typing import Dict, Union

from lxml import etree as et

from eml.types import _NoTagObject


class ExtensionString(_NoTagObject):
    """
    Extension string representing a string value with extra attributes.

    Parameters
    ----------
    value : str | ExtensionString
        Value of the string or another ExtensionString object.
    system : str, optional
        The data management system.
    """
    def __init__(self, value: Union[str, ExtensionString], system: str = None) -> None:
        super().__init__()
        if isinstance(value, ExtensionString):
            self.__value__ = value
            self.__system__ = system
        else:
            self.__value__ = value
            self.__system__ = system
        return

    @property
    def system(self) -> str:
        """str: The data management system within which an identifier is in scope and therefore unique."""
        return self.__system__

    def __str__(self) -> str:
        return self.__value__

    def __repr__(self) -> str:
        extension = ""
        if self.__system__ is not None:
            extension = f"system='{self.__system__}' "
        return f"<ExtensionString {extension}({self.__value__})>"

    def __eq__(self, other: Union[str, ExtensionString]) -> bool:
        if isinstance(other, str):
            return self.__value__ == other
        elif isinstance(other, ExtensionString):
            return self.__value__ == other.__value__ and self.system == other.system
        else:
            return False

    @classmethod
    def parse(cls, element: et.Element, nmap: Dict) -> ExtensionString | None:
        """
        Parses a lxml element into an Extension String.

        Parameters
        ----------
        element : lxml.tree.Element
            Object in the Element format to be parsed into a Python object.
        nmap : Dict
            Dictionary of namespace.

        Returns
        -------
        ExtensionString
            Object parsed.
        """
        if element is None:
            return None
        return ExtensionString(element.text, system=element.get("system", None))

    def to_element(self) -> et.Element:
        """
        Generate a lxml.tree.Element from the extended string.

        Returns
        -------
        lxml.tree.Element
            Object in the XML Element format.

        Raises
        ------
        RuntimeError
            If tag is not set before calling this method.
        """
        element = super().to_element()
        if self.__system__ is not None:
            element.set("system", self.__system__)
        element.text = self.__value__
        return element



import os.path
import unittest

from dwca.classes import OutsideClass
from dwca.terms import TaxonID
from test_dwca_classes.test_outside_class_common import TestOutsideCommon

PATH = os.path.abspath(os.path.dirname(__file__))


class TestOutside(TestOutsideCommon):
    def setUp(self) -> None:
        self.outside_xml, self.text = self.read_xml(os.path.join(PATH, os.pardir, "example_data", "meta.xml"))
        return

    def test_name(self):
        outside_class = OutsideClass.from_string(self.text)
        self.assertEqual("SpeciesProfile", outside_class.name, "Name parsed incorrectly.")

    def test_parse(self):
        super().__test_parse__()

    def test_none(self):
        super().__test_none__()

    def test_merge(self):
        super().__test_merge__()

    def test_missing_field(self):
        self.assertRaises(
            AssertionError,
            OutsideClass.from_string,
            """
<core>
    <files>
      <location>missing.txt</location>
    </files>
    <id index="0"/>
    <field index="0" term=""/>
    <field index="1" term=""/>
    <field index="3" term=""/>
</core>
            """
        )

    def test_field_twice(self):
        self.assertRaises(
            AssertionError,
            OutsideClass.from_string,
            """
<core>
    <files>
      <location>missing.txt</location>
    </files>
    <id index="0"/>
    <field index="0" term=""/>
    <field index="1" term=""/>
    <field index="3" term=""/>
    <field index="2" term=""/>
    <field index="3" term=""/>
    <field index="4" term=""/>
</core>
            """
        )

    def test_parse_invalid(self):
        self.assertRaises(
            AssertionError,
            OutsideClass.from_string,
            '<extension />'
        )
        self.assertRaises(
            KeyError,
            OutsideClass.from_string,
            """
<core-id>
    <field index="0" term=""/>
</core-id>
            """
        )

    def test_set_primary_key(self):
        self.__test_set_primary_key__()

    def test_sql_table(self):
        self.__test_sql_table__()

    def test_insert(self):
        self.__test_insert__()

    def test_insert_lazy(self):
        text_1 = """
        <extension encoding="UTF-8" linesTerminatedBy="\\n" fieldsTerminatedBy="\\t" fieldsEnclosedBy="" ignoreHeaderLines="1" rowType="http://example.org/terms/extension1">
        <files>
            <location>extension1.txt</location>
        </files>
        <coreid index="0" /><field index="1" term="http://example.org/terms/column1"/>
        <field index="2" term="http://example.org/terms/column2"/>
        </extension>
        """
        extension1 = OutsideClass.from_string(text_1)
        with self.assertRaisesRegex(
                RuntimeError,
                "Extension",
        ):
            next(extension1.insert_sql)
        extension1.set_core_field(TaxonID(5))
        with open(
                os.path.join(PATH, os.pardir, "example_data", "extension_example1.txt"), "rb"
        ) as file:
            extension1.read_file("", source_file=file, lazy=True)
        text_2 = """
        <extension encoding="UTF-8" linesTerminatedBy="\\n" fieldsTerminatedBy="\\t" fieldsEnclosedBy="" ignoreHeaderLines="1" rowType="http://example.org/terms/extension2">
        <files>
          <location>extension2.txt</location>
        </files>
        <coreid index="0" /><field index="1" term="http://example.org/terms/column1"/>
        </extension>
        """
        extension2 = OutsideClass.from_string(text_2)
        extension2.set_core_field(TaxonID(5))
        with open(
                os.path.join(PATH, os.pardir, "example_data", "extension_example2.txt"), "rb"
        ) as file:
            extension2.read_file("", source_file=file, lazy=True)
        expected_values_1 = [
            ("urn:lsid:example.org:taxname:0", "true", "false"),
            ("urn:lsid:example.org:taxname:8", "false", "true"),
            ("urn:lsid:example.org:taxname:19", "true", "true")
        ]
        self.assertEqual(3, len(list(extension1.insert_sql)), "Not the right amount of rows.")
        for (statement, values), expected in zip(extension1.insert_sql, expected_values_1):
            self.assertEqual(
                self.normalize_sql("""
                        INSERT INTO "extension1" (
                            "taxonID",
                            "column1",
                            "column2"
                        ) VALUES (%s, %s, %s)
                        """), self.normalize_sql(statement), "Statement do not match"
            )
            self.assertEqual(expected, values, "Values do not match")
        extension1.close()
        expected_values_2 = [
            ("urn:lsid:example.org:taxname:20", "true"),
            ("urn:lsid:example.org:taxname:27", "true"),
            ("urn:lsid:example.org:taxname:41", "false")
        ]
        self.assertEqual(3, len(list(extension2.insert_sql)), "Not the right amount of rows.")
        for (statement, values), expected in zip(extension2.insert_sql, expected_values_2):
            self.assertEqual(
                self.normalize_sql("""
                        INSERT INTO "extension2" (
                            "taxonID",
                            "column1"
                        ) VALUES (%s, %s)
                        """), self.normalize_sql(statement), "Statement do not match"
            )
            self.assertEqual(expected, values, "Values do not match")
        extension2.close()


if __name__ == '__main__':
    unittest.main()
